-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ --
-- GreenFert
--
-- Purpose: Chop Oilseed Radish up to act as green fertilizer
-- 
-- Authors: Timmiej93
--
-- Copyright (c) Timmiej93, 2018
--
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ --

GreenFert = {}

function GreenFert.initSpecialization()
    WorkArea.registerAreaType("greenfert")
end

function GreenFert.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(WorkArea, specializations)
end

function GreenFert:preLoad(savegame)
	self.loadWorkAreaFromXML = Utils.overwrittenFunction(self.loadWorkAreaFromXML, GreenFert.loadWorkAreaFromXML)
end

function GreenFert:load(savegame)

    self.doCheckSpeedLimit = Utils.overwrittenFunction(self.doCheckSpeedLimit, GreenFert.doCheckSpeedLimit)
    self.processGreenFertAreas = GreenFert.processGreenFertAreas

    if next(self.groundReferenceNodes) == nil then
        print("Warning: No ground reference nodes in  "..self.configFileName)
    end

    self.onlyActiveWhenLowered = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.onlyActiveWhenLowered#value"), true)
    
    self.startActivationTimeout = 2000
    self.startActivationTime = 0
    self.hasGroundContact = false
    self.doGroundManipulation = false
    self.isSpeedLimitActive = false
    self.showFieldNotOwnedWarning = false

    self.groundContactFlag = self:getNextDirtyFlag()
end

function GreenFert:readUpdateStream(streamId, timestamp, connection)
    if (connection:getIsServer()) then
        self.hasGroundContact = streamReadBool(streamId)
        self.showFieldNotOwnedWarning = streamReadBool(streamId)
    end
end

function GreenFert:writeUpdateStream(streamId, connection, dirtyMask)
    if (not connection:getIsServer()) then
        streamWriteBool(streamId, self.hasGroundContact)
        streamWriteBool(streamId, self.showFieldNotOwnedWarning)
    end
end

function GreenFert:updateTick(dt)
    
    self.isSpeedLimitActive = false

    if (self:getIsActive()) then
        local showFieldNotOwnedWarning = false

        if (self.isServer) then
            local hasGroundContact = self:getIsTypedWorkAreaActive(WorkArea.AREATYPE_GREENFERT)

            if (self.hasGroundContact ~= hasGroundContact) then
                self:raiseDirtyFlags(self.groundContactFlag)
                self.hasGroundContact = hasGroundContact
            end
        end

        local hasGroundContact = self.hasGroundContact
        self.doGroundManipulation = (hasGroundContact and (not self.onlyActiveWhenLowered or self:isLowered(false)) and self.startActivationTime <= g_currentMission.time)
        
        if (self.doGroundManipulation) then
            self.isSpeedLimitActive = true
            local workAreas, showWarning, _ = self:getTypedNetworkAreas(WorkArea.AREATYPE_GREENFERT, true)
            if (self.isServer) then
                showFieldNotOwnedWarning = showWarning
                if (table.getn(workAreas) > 0) then
                    self:processGreenFertAreas(workAreas)
                end
                g_currentMission.missionStats:updateStats("workedTime", dt/(1000*60))
            end
        end
        -- if (self.isClient) then
        --     if self.doGroundManipulation and self:getLastSpeed() > 3 then
        --         if self:getIsActiveForSound() then
        --             SoundUtil.playSample(self.sampleRoller, 0, 0, nil)
        --         end
        --     else
        --         SoundUtil.stopSample(self.sampleRoller)
        --     end
        -- end
        if (self.isServer) then
            if showFieldNotOwnedWarning ~= self.showFieldNotOwnedWarning then
                self.showFieldNotOwnedWarning = showFieldNotOwnedWarning
                self:raiseDirtyFlags(self.groundContactFlag)
            end
        end
    end
end

function GreenFert:draw()
    if (self.showFieldNotOwnedWarning) then
        g_currentMission:showBlinkingWarning(g_i18n:getText("warning_youDontOwnThisField"))
    end
end

function GreenFert:onDeactivate()
    self.showFieldNotOwnedWarning = false
end

function GreenFert:doCheckSpeedLimit(superFunc)
    local parent = false
    if (superFunc ~= nil) then
        parent = superFunc(self)
    end
    return parent or self.isSpeedLimitActive
end

function GreenFert:getDoGroundManipulation(superFunc)
    if (not self.doGroundManipulation) then
        return false
    end
    if (superFunc ~= nil) then
        return superFunc(self)
    end
    return true
end

function GreenFert:loadWorkAreaFromXML(superFunc, workArea, xmlFile, key)
    local retValue = true
    if (superFunc ~= nil) then
        retValue = superFunc(self, workArea, xmlFile, key)
    end
    if (workArea.type == WorkArea.AREATYPE_DEFAULT) then
        workArea.type = WorkArea.AREATYPE_GREENFERT
    end
    return retValue
end

function GreenFert:processGreenFertAreas(workAreas)
	local numAreas = table.getn(workAreas)
    local areaSum = 0
    for i=1, numAreas do
        local x = workAreas[i][1]
        local z = workAreas[i][2]
        local x1 = workAreas[i][3]
        local z1 = workAreas[i][4]
        local x2 = workAreas[i][5]
        local z2 = workAreas[i][6]

        local detailId = g_currentMission.terrainDetailId
        local x0,z0, widthX,widthZ, heightX,heightZ = Utils.getXZWidthAndHeight(detailId, x,z, x1,z1, x2,z2)

        local fruitIdOil = g_currentMission.fruits[FruitUtil.FRUITTYPE_OILSEEDRADISH].id
        local fruitIdBeet = g_currentMission.fruits[FruitUtil.FRUITTYPE_SUGARBEET].id

        -- Add fertilization layer
        addDensityMaskedParallelogram(
            detailId,
            x0,z0, widthX,widthZ, heightX,heightZ,
            g_currentMission.sprayLevelFirstChannel,
            g_currentMission.sprayLevelNumChannels,
            fruitIdOil, 
            1, 1,
            1
        )

        -- Set visual spray level
        setDensityMaskedParallelogram(
            detailId, 
            x0,z0, widthX,widthZ, heightX,heightZ,
            g_currentMission.sprayFirstChannel,
            g_currentMission.sprayNumChannels,
            fruitIdOil,
            1, 1,
            1
        )

        -- Change the oilseed radish to chopped beet greens
        setDensityMaskedParallelogram(
            fruitIdBeet,
            x0,z0, widthX,widthZ, heightX,heightZ,
            0, 9,
            fruitIdOil, 
            1, 1,
            511
        )
    end
end

function GreenFert:update(dt)end
function GreenFert:delete()end
function GreenFert:keyEvent(unicode, sym, modifier, isDown)end
function GreenFert:mouseEvent(posX, posY, isDown, isUp, button)end
